#!/usr/bin/perl
#cvrtGM.pl
#Wei Zhu, Iowa State University, 2001.

use Getopt::Std;

my $label = "GM";
my %opts = ();
getopts('s:o:h',\%opts);

if( $opts{h}){
    print "Usage: $0 [-o GIIF-output-file] [-s Seq_Shift] <original-GeneMark.hmm-output-file> \n";
    exit -1;
}


my $outfile = $opts{o} || "-";
my $SeqShift = $opts{s} || 0;
my $infile = $ARGV[0] || "-";
#print "\n $infile $outfile $SeqShift \n";


open FIN,"$infile" or die "Could not open file $infile";
open FOUT,">$outfile" or die "Could not open file $outfile";

while( $line = <FIN> ){
	$line = <FIN> until($line =~ /^Sequence name/ || $line eq "");
	last if($line eq "");

	$line =~ /^Sequence name: (\w+)/; 
	$id = $1;
	$line = <FIN>;
	$line =~ /^Sequence length: (\d+) bp/;
	$length = $1;
#Print title:
	printf FOUT "File=$infile\n";
	printf FOUT "SeqID=$id\n";
	printf FOUT "GenBank=N\n";
	printf FOUT "SeqShift=$SeqShift\n";
	printf FOUT "Length=$length\n";
	printf FOUT "Source=GeneMark.hmm\n";
	printf FOUT "Label=%s\n",$label;
	printf FOUT "SourceID=3\n";
	printf FOUT "HasIntronScore=N\n";
	printf FOUT "HasExonScore=N\n";
	printf FOUT "\nBEGIN\n";

	$line = <FIN> until($line =~ /^Predicted/);
	$count = 0;
	$line = <FIN>;
	undef(@geneSet);
	undef(@dataSet);
	DONE: while(!eof && $line !~ /^ Input/){
#print "$line";
		$count++;
		$line = <FIN> until($line =~ /^ *$count / || $line =~ /^ Input/);
#print "$line";
		if($line =~ /^ Input/){ last DONE; }
		undef(@set);
		undef($data);
		do{
			$data = $data.$line;
			$line =~ s/(^\s+|\s+$)//g;
                	@line = split(/ +/,$line);
			$signal = $line[2];
                	if($signal eq "-"){
			  push @set, [$line[5]+$SeqShift,$line[4]+$SeqShift];
                	}
			else{
			  push @set, [$line[4]+$SeqShift,$line[5]+$SeqShift];
                	}
			$line = <FIN>;
		}while($line =~ /^ *$count / );
		if($signal eq "-"){
			@set = reverse(@set);
		}
		push @geneSet, [@set];
		push @dataSet, $data;
	}
	$line = <FIN> until(eof || $line =~ /^>/);
	$total = @geneSet;
	for($i=0;$i<$total;$i++){
#Print data:
		$info = $geneSet[$i];
		$num = 0;
		printf FOUT "***%s %3d***\n",$label, $i + 1;
		printf FOUT "  1)Structure:   ( .Begin.  ..End..)\n";
		foreach $exon (@$info){
			$num++;
			($beg,$end) = @$exon;
			printf FOUT "    Exon %3d:      %7d  %7d\n", $num,$beg,$end; 
		}
		printf FOUT "  2)Information:\n";
		@data = split(/\n/,$dataSet[$i]);
		foreach $infoLine (@data){
			printf FOUT "    $infoLine\n";
		}
		if(!eof){
			do{
				printf FOUT "    $line";
				$line = <FIN>;
			}until($line =~ /^\s/ || $line eq "");
			last if ($line eq "");
			$line = <FIN>;
		}
	}

	print FOUT "\nEND\n";
	last;
}

sub convert{
	$pd /= 100.0;
	$pa /= 100.0;
	$pd = check($pd);
	$pa = check($pa);
	$score = check($score);
}

sub check{
	my($x) = @_;
	$x = ($x<0)?0:$x;
	$x = ($x>=10)?9.999:$x;
}
