#!/usr/bin/perl
#cvrtGSN.pl
#Wei Zhu, Iowa State University, 2001.

use Getopt::Std;

my $label="GSN";

my %opts = ();
getopts('s:o:h',\%opts);

if( $opts{h}){
    print "Usage: $0 [-o GIIF-output-file] [-s Seq_Shift] <original-GENSCAN-output-file> \n";
    exit -1;
}


my $outfile = $opts{o} || "-";
my $SeqShift = $opts{s} || 0;
my $infile = $ARGV[0] || "-";
#print "\n $infile $outfile $SeqShift \n";


open FIN,"$infile" or die "Could not open file $infile";
open FOUT,">$outfile" or die "Could not open file $outfile";


while( $line = <FIN> ){
	$line = <FIN> until($line =~ /^Sequence/ || $line eq "");
	last if($line eq "");

	$line =~ /^Sequence (.+) : (\d+) bp :/; 
	$id = $1;
	$length = $2;
#Print title:
	printf FOUT "File=$infile\n";
	printf FOUT "SeqID=$id\n";
	printf FOUT "GenBank=N\n";
	printf FOUT "SeqShift=$SeqShift\n";
	printf FOUT "Length=$length\n";
	printf FOUT "Source=GENSCAN\n";
	printf FOUT "Label=%s\n",$label;
	printf FOUT "SourceID=1\n";
	printf FOUT "HasIntronScore=Y\n";
	printf FOUT "HasExonScore=Y\n";
	printf FOUT "\nBEGIN\n";

	$line = <FIN> until($line =~ /^Parameter/);
	$count = 0;
	$line = <FIN>;
	undef(@geneSet);
	undef(@dataSet);
	DONE: while($line !~ /^Predicted peptide/){
#print "$line";
		$count++;
		$line = <FIN> until($line =~ /^.?$count/ || $line =~ /^Predicted peptide/);
#print "$line";
		if($line =~ /^Predicted peptide/){ last DONE; }
		undef(@set);
		undef($data);
		do{
			$data = $data.$line;
			if($line !~ /PlyA|Prom/ ){
				$line =~ /^.?\d+\.\d+ (.*)/;
#printf $1."\n";
				@line = split(/ +/,$1);
				$signal = $line[1];
				push @set, [$line[2]+$SeqShift,$line[3]+$SeqShift,$line[7],$line[8],$line[10]];
#printf "@line";
			}
			$line = <FIN>;
		}while($line =~ /^.?$count/ );
		if($signal eq "-"){
			@set = reverse(@set);
		}
		push @geneSet, [@set];
		push @dataSet, $data;
	}

	$line = <FIN> until($line =~ /^>/);
	$total = @geneSet;
	for($i=0;$i<$total;$i++){
#Print data:
		$info = $geneSet[$i];
		$num = 0;
		printf FOUT "***%s %3d***\n",$label, $i + 1;
		printf FOUT "  1)Structure:   ( .Begin.  ..End..   .Do.   .Ac.   Score )\n";
		foreach $exon (@$info){
			$num++;
			($beg,$end,$pd,$pa,$score) = @$exon;
			&convert();
			printf FOUT "    Exon %3d:      %7d  %7d   %4.2f   %4.2f   %5.3f\n", $num,$beg,$end, $pd, $pa, $score; 
		}
		printf FOUT "  2)Information:\n";
		@data = split(/\n/,$dataSet[$i]);
		foreach $infoLine (@data){
			printf FOUT "    $infoLine\n";
		}
		do{
			printf FOUT "    $line";
			$line = <FIN>;
		}until($line =~ /^\s/ || $line eq "");
		last if ($line eq "");
		$line = <FIN>;
	}

	printf FOUT "END\n";
	last;
}

sub convert{
	$pd /= 100.0;
	$pa /= 100.0;
	$pd = check($pd);
	$pa = check($pa);
	$score = check($score);
}

sub check{
	my($x) = @_;
	$x = ($x<0)?0:$x;
	$x = ($x>=10)?9.999:$x;
}
